@@ -19,9 +19,9 @@ pub trait Diff {
     ///
     /// If a line is added (+) then old_line is 0 and new_line is the line in the destination file.
     ///
-    /// If a line is added (-) then old_line is the line in the source file and new_line is 0.
+    /// If a line is removed (-) then old_line is the line in the source file and new_line is 0.
     ///
-    /// If a line is added ( ) then old_line is the line in the source file and new_line is the line in the destination file.
+    /// If a line is nothing ( ) then old_line is the line in the source file and new_line is the line in the destination file.
     fn add_line(&mut self, old_line: u32, new_line: u32, line: &[u8]);
 
     /// A new hunk is created
@@ -113,7 +113,7 @@ impl<'a> LineReader<'a> {
     }
 
     fn is_triple_minus(&self) -> bool {
-        self.buf.starts_with(b"---")
+        self.buf.starts_with(b"--- ")
     }
 
     fn is_index(&self) -> bool {
@@ -138,32 +138,41 @@ impl<'a> LineReader<'a> {
         }
     }
 
-    pub fn parse_numbers(&self) -> (u32, u32) {
+    pub fn parse_numbers(&self) -> (u32, u32, u32, u32) {
         // we know that line is beginning with "@@ -"
         let buf = unsafe { self.buf.get_unchecked(4..) };
-        let mut x = 0;
-        let mut y = 0;
-        let mut iter = buf.iter();
-        while let Some(c) = iter.next() {
-            if *c >= b'0' && *c <= b'9' {
-                x = x * 10 + u32::from(*c - b'0');
-            } else {
-                break;
-            }
+
+        // NUMS_PAT = re.compile(r'^@@ -([0-9]+),?([0-9]+)? \+([0-9]+),?([0-9]+)? @@')
+        let iter = &mut buf.iter();
+        let x = iter
+            .take_while(|&&c| c >= b'0' && c <= b'9')
+            .fold(0, |r, &c| r * 10 + u32::from(c - b'0'));
+
+        let c = *iter.next().unwrap();
+        let y = if c >= b'0' && c <= b'9' {
+            iter.take_while(|&&c| c >= b'0' && c <= b'9')
+                .fold(u32::from(c - b'0'), |r, &c| r * 10 + u32::from(c - b'0'))
+        } else {
+            1
+        };
+
+        if c != b'+' {
+            iter.skip_while(|&&c| c != b'+').next();
         }
-        while let Some(c) = iter.next() {
-            if *c == b'+' {
-                break;
-            }
-        }
-        for c in iter {
-            if *c >= b'0' && *c <= b'9' {
-                y = y * 10 + u32::from(*c - b'0');
-            } else {
-                break;
-            }
-        }
-        (x, y)
+
+        let z = iter
+            .take_while(|&&c| c >= b'0' && c <= b'9')
+            .fold(0, |r, &c| r * 10 + u32::from(c - b'0'));
+
+        let c = *iter.next().unwrap();
+        let t = if c >= b'0' && c <= b'9' {
+            iter.take_while(|&&c| c >= b'0' && c <= b'9')
+                .fold(u32::from(c - b'0'), |r, &c| r * 10 + u32::from(c - b'0'))
+        } else {
+            1
+        };
+
+        (x, y, z, t)
     }
 
     fn get_filename(buf: &[u8]) -> &str {
@@ -228,15 +237,9 @@ impl<'a> LineReader<'a> {
         // we know that line is beginning with "old/new mode "
         // the following number is an octal number with 6 digits (so max is 8^6 - 1)
         let buf = unsafe { self.buf.get_unchecked(start.len()..) };
-        let mut x = 0;
-        for c in buf.iter() {
-            if *c >= b'0' && *c <= b'7' {
-                x = x * 8 + u32::from(*c - b'0');
-            } else {
-                break;
-            }
-        }
-        x
+        buf.iter()
+            .take_while(|&&c| c >= b'0' && c <= b'7')
+            .fold(0, |r, &c| r * 8 + u32::from(c - b'0'))
     }
 }
 
@@ -446,18 +449,22 @@ impl<'a> PatchReader<'a> {
     }
 
     fn parse_hunks<D: Diff>(&mut self, line: &mut LineReader, diff: &mut D) {
-        let (o1, n1) = line.parse_numbers();
-        self.parse_hunk(o1, n1, diff);
+        let (o1, o2, n1, n2) = line.parse_numbers();
+        self.parse_hunk(o1, o2, n1, n2, diff);
         while let Some(line) = self.next(PatchReader::hunk_at, true) {
-            let (o1, n1) = line.parse_numbers();
-            self.parse_hunk(o1, n1, diff);
+            let (o1, o2, n1, n2) = line.parse_numbers();
+            self.parse_hunk(o1, o2, n1, n2, diff);
         }
     }
 
-    fn parse_hunk<D: Diff>(&mut self, o: u32, n: u32, diff: &mut D) {
-        let mut old_count = o;
-        let mut new_count = n;
-
+    fn parse_hunk<D: Diff>(
+        &mut self,
+        mut old_count: u32,
+        mut old_lines: u32,
+        mut new_count: u32,
+        mut new_lines: u32,
+        diff: &mut D,
+    ) {
         diff.new_hunk();
 
         while let Some(line) = self.next(PatchReader::hunk_change, true) {
@@ -467,18 +474,25 @@ impl<'a> PatchReader<'a> {
                 b'-' => {
                     diff.add_line(old_count, 0, unsafe { line.buf.get_unchecked(1..) });
                     old_count += 1;
+                    old_lines -= 1;
                 }
                 b'+' => {
                     diff.add_line(0, new_count, unsafe { line.buf.get_unchecked(1..) });
                     new_count += 1;
+                    new_lines -= 1;
                 }
                 b' ' => {
                     diff.add_line(old_count, new_count, unsafe { line.buf.get_unchecked(1..) });
                     old_count += 1;
                     new_count += 1;
+                    old_lines -= 1;
+                    new_lines -= 1;
                 }
                 _ => {}
             }
+            if old_lines == 0 && new_lines == 0 {
+                break;
+            }
         }
     }
 
@@ -531,15 +545,9 @@ impl<'a> PatchReader<'a> {
     }
 
     fn parse_usize(buf: &[u8]) -> usize {
-        let mut x = 0;
-        for c in buf.iter() {
-            if *c >= b'0' && *c <= b'9' {
-                x = x * 10 + usize::from(*c - b'0');
-            } else {
-                break;
-            }
-        }
-        x
+        buf.iter()
+            .take_while(|&&c| c >= b'0' && c <= b'9')
+            .fold(0, |r, &c| r * 10 + usize::from(c - b'0'))
     }
 
     fn skip_binary(&mut self) -> Vec<BinaryHunk> {
@@ -607,15 +615,15 @@ mod tests {
     #[test]
     fn test_numbers() {
         let cases = [
-            ("@@ -123,456 +789,101112 @@", (123, 789)),
-            ("@@ -123 +789,101112 @@", (123, 789)),
-            ("@@ -123,456 +789 @@", (123, 789)),
-            ("@@ -123 +789 @@", (123, 789)),
+            ("@@ -123,456 +789,101112 @@", (123, 456, 789, 101112)),
+            ("@@ -123 +789,101112 @@", (123, 1, 789, 101112)),
+            ("@@ -123,456 +789 @@", (123, 456, 789, 1)),
+            ("@@ -123 +789 @@", (123, 1, 789, 1)),
         ];
         for c in cases.iter() {
             let buf = c.0.as_bytes();
             let line = LineReader { buf: &buf };
-            assert!(line.parse_numbers() == c.1);
+            assert_eq!(line.parse_numbers(), c.1);
         }
     }
 
